/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JavaScript Engine testing utilities.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Seno.Aiko@gmail.com
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//-----------------------------------------------------------------------------
var BUGNUMBER = 310351;
var summary = 'Convert host "list" objects to arrays';
var actual = '';
var expect = '';

printBugNumber(BUGNUMBER);
printStatus (summary);

var nodeList = [];
if (typeof document != 'undefined')
{
  nodeList = document.getElementsByTagName('*');
}
else
{
  printStatus('test using dummy array since no document available');
}
 
var array = Array.prototype.slice.call(nodeList, 0);

expect = 'Array';
actual = array.constructor.name;

// nodeList is live and may change
var saveLength = nodeList.length;

reportCompare(expect, actual, summary + ': constructor test');

expect = saveLength;
actual = array.length;

reportCompare(expect, actual, summary + ': length test');
expect = true;
actual = true;

for (var i = 0; i < saveLength; i++)
{
  if (array[i] != nodeList[i])
  {
    actual = false;
    summary += ' Comparison failed: array[' + i + ']=' + array[i] +
      ', nodeList[' + i + ']=' + nodeList[i];
    break;
  }
}

reportCompare(expect, actual, summary + ': identical elements test');

