/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== msgCompSMIMEOverlay.css ========================================
  == Styles for the S/Mime in composer window.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

#button-security {
  list-style-image: url("chrome://messenger/skin/smime/icons/smbtn1.gif");
  -moz-image-region: rect(0px 49px 33px 0px);
}

#button-security:hover {
  -moz-image-region: rect(0px 99px 33px 50px);
} 

#button-security:hover:active {
  -moz-image-region: rect(0px 149px 33px 100px);
} 

#button-security[disabled] {
  -moz-image-region: rect(0px 199px 33px 150px) !important;
} 


#msgcomposeWindow #signing-status {
  list-style-image: none;
  visibility: collapse;
}

#msgcomposeWindow[signing="ok"] #signing-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbSignOk.gif");
  visibility: visible;
}

#msgcomposeWindow[signing="notok"] #signing-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbSignNotOk.gif");
  visibility: visible;
}


#msgcomposeWindow #encryption-status {
  list-style-image: none;
  visibility: collapse;
}

#msgcomposeWindow[crypto="ok"] #encryption-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbCryptoOk.gif");
  visibility: visible;
}

#msgcomposeWindow[crypto="notok"] #encryption-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbCryptoNotOk.gif");
  visibility: visible;
}
